from track2p.gui.t2p_wd import Track2pWindow
from track2p.gui.import_wd import ImportWindow
from track2p.gui.raster_wd import RasterWindow
#from track2p.gui.s2p_wd import Suite2pWindow

class WindowManager:
    def __init__(self, main_window):
        self.main_window = main_window
        self.t2p_window = Track2pWindow(self.main_window)
        self.import_window = ImportWindow(self.main_window)
        self.raster_window=RasterWindow(self.main_window)
        #self.suite2p_window=Suite2pWindow(self.main_window)

    def open_track2p_wd(self):
        self.t2p_window.show()

    def open_import_wd(self):
        self.import_window.show()
        
    def open_raster_wd(self):
        self.raster_window.show()
        
        
        
    #def open_suite2p_wd(self):
    #   self.suite2p_window.show()
