import os

# make a directory based on path if it doesn't exist yet

def make_dir(path):
    if not os.path.exists(path):
        os.makedirs(path)
        print('Created directory: ' + path)
    else:
        print('Directory already exists: ' + path)